package com.fluxtest.controller;

import io.opentelemetry.api.baggage.Baggage;
import io.opentelemetry.context.Context;
import io.opentelemetry.context.Scope;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.filter.AbstractRequestLoggingFilter;

import static com.fluxtest.controller.CommonUtil.*;

@Configuration
@Slf4j
public class ReqLogConfig {

    @Bean
    public AbstractRequestLoggingFilter requestLoggingFilter() {
        RequestLoggingFilter filter = new RequestLoggingFilter();
        filter.setIncludeQueryString(true);
        filter.setIncludePayload(true);
        filter.setMaxPayloadLength(10000);
        filter.setIncludeHeaders(true);
        filter.setIncludeClientInfo(true);
        log.info("added request log filter");
        return filter;
    }

    private static class RequestLoggingFilter extends AbstractRequestLoggingFilter {

        public RequestLoggingFilter() {
            setAfterMessagePrefix("REQUEST: [");
        }

        @Override
        protected void beforeRequest(HttpServletRequest request, String message) {
            Baggage baggage = Baggage.builder()
                    .put(BAGGAGE_USER_ID, String.valueOf(System.currentTimeMillis()))
                    .build();
            Scope scope = baggage.storeInContext(Context.current()).makeCurrent();
            request.setAttribute(SCOPE, scope);
            print("ReqLogConfig.beforeRequest", true);
        }

        @Override
        protected void afterRequest(HttpServletRequest request, String message) {
            //print("ReqLogConfig.afterRequest", false);
            log.info("afterRequest: message:[{}] traceId: {}", message, getTraceId(false));
            Scope scope = (Scope) request.getAttribute(SCOPE);
            if (scope != null) {
                log.info("close scope: {} {}", scope.getClass(), scope);
                scope.close();
            }
            print("afterRequest finish", false);
            //Span.current().end();
        }
    }
}
