package com.fluxtest.controller;

import io.opentelemetry.api.baggage.Baggage;
import io.opentelemetry.instrumentation.annotations.WithSpan;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.codec.ServerSentEvent;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;

import java.time.Duration;

import static com.fluxtest.controller.CommonUtil.BAGGAGE_USER_ID;


@Slf4j
@Service
public class TestService {

    @WithSpan
    public String getBlockResult(String req) {
        String userIdStr = Baggage.current().getEntryValue(BAGGAGE_USER_ID);
        String id = String.valueOf(System.currentTimeMillis());
        return "block_" + req + "_" + id + "_" + userIdStr + "#";
    }

    @WithSpan
    public Flux<ServerSentEvent<String>> getStreamResult(String req) {
        // 创建一个Flux，每秒发送一次事件
        Flux<Long> tickFlux = Flux.interval(Duration.ofSeconds(1));
        // 将Flux<Long>转换为Flux<ServerSentEvent<String>>
        return tickFlux.take(1).map(sequence -> {
            String userIdStr = Baggage.current().getEntryValue(BAGGAGE_USER_ID);
            String id = String.valueOf(System.currentTimeMillis());
            String tag = "sendStream_" + req + "_" + id + "_" + userIdStr;
            CommonUtil.print(tag, false);
            return ServerSentEvent.builder("Message: " + sequence).id(tag).build();
        });
    }

}
